// (c) olaf schnabel, institute of cartography, eth zurich, 2004
// adapted to the map symbol brewer
// feedback: schnabel@karto.baug.ethz.ch
// description: textbox for numbers

/* requirements

// variables:
var svgNS = "http://www.w3.org/2000/svg";

// a background object is necessary for proper working of the event handlers
// a group for appending the textbox
*/

//textbox properties
// controlObjectId = 0: no control object (e.g. buttons or slider) for textbox,
// controlObjectId = object name as string (in quotes): textbox can be controlled with object
function textbox(x,y,boxWidth,boxHeight,rx,ry,defaultVal,minBorder,maxBorder,boxCSSclass,textCSSclass,cursorCSSclass,groupId,controlObjectId) {
	this.x = x;
	this.y = y;
	this.boxWidth = boxWidth;
	this.boxHeight = boxHeight;
	this.rx = rx;
	this.ry = ry;
	this.defaultVal = defaultVal;
	this.minBorder = minBorder;
	this.maxBorder = maxBorder;
	this.boxCSSclass = boxCSSclass;
	this.textCSSclass = textCSSclass;
	this.cursorCSSclass = cursorCSSclass;
	this.groupId = groupId;
	this.textboxGroup = document.getElementById(this.groupId);
	this.controlObjectId = controlObjectId;
	this.createTextbox();
	this.textboxStatus = 0;
}

//create textbox
textbox.prototype.createTextbox = function() {
	var textboxRect = document.createElementNS(svgNS,"rect");
	textboxRect.setAttributeNS(null,"x",(this.x-this.boxWidth/2));
	textboxRect.setAttributeNS(null,"y",(this.y-this.boxHeight/2));
	textboxRect.setAttributeNS(null,"width",this.boxWidth);
	textboxRect.setAttributeNS(null,"height",this.boxHeight);
	textboxRect.setAttributeNS(null,"rx",this.rx);
	textboxRect.setAttributeNS(null,"ry",this.ry);
	textboxRect.setAttributeNS(null,"class",this.boxCSSclass);
	textboxRect.setAttributeNS(null,"id",this.groupId+"Field");
	this.textboxGroup.appendChild(textboxRect);
	
	var textboxText = document.createElementNS(svgNS,"text");
	textboxText.setAttributeNS(null,"x",(this.x-this.boxWidth/2+2));
	textboxText.setAttributeNS(null,"y",(this.y+this.boxHeight/2-3));
	textboxText.setAttributeNS(null,"class",this.textCSSclass);
	textboxText.setAttributeNS(null,"id",this.groupId+"Text");
	textboxText.setAttributeNS(null,"pointer-events","none");
	var textboxTextContent = document.createTextNode(this.defaultVal);
	textboxText.appendChild(textboxTextContent);
	this.textboxGroup.appendChild(textboxText);

	var textboxCursor = document.createElementNS(svgNS,"line");
	textboxCursor.setAttributeNS(null,"x1",(this.x-this.boxWidth/2+3));
	textboxCursor.setAttributeNS(null,"y1",(this.y-this.boxHeight/2+2));
	textboxCursor.setAttributeNS(null,"x2",(this.x-this.boxWidth/2+3));
	textboxCursor.setAttributeNS(null,"y2",(this.y+this.boxHeight/2-2));
	textboxCursor.setAttributeNS(null,"class",this.cursorCSSclass);
	textboxCursor.setAttributeNS(null,"id",this.groupId+"Cursor");
	textboxCursor.setAttributeNS(null,"visibility","hidden");
	this.textboxGroup.appendChild(textboxCursor);
	
	// add click event listener to the textbox group
	this.textboxGroup.addEventListener("click",this,false);

}

//remove all textbox elements
textbox.prototype.removeTextbox = function() {
    var textboxRect = document.getElementById(this.groupId+"Field");
	this.textboxGroup.removeChild(textboxRect);
    var textboxText = document.getElementById(this.groupId+"Text");
	this.textboxGroup.removeChild(textboxText);
    var textboxCursor = document.getElementById(this.groupId+"Cursor");
	this.textboxGroup.removeChild(textboxCursor);
    this.textboxGroup.removeEventListener("click",this,false);
}

//click on and write in textbox
textbox.prototype.handleEvent = function(evt) {

         // get text node and value
         var textNode = document.getElementById(this.groupId+"Text");
	 var textVal = textNode.firstChild.nodeValue;
         var cursor = document.getElementById(this.groupId+"Cursor");


	if (evt.type == "click")
	{

          if (evt.currentTarget.nodeName == "svg" && this.textboxStatus == 2)
          {
		// set textbox status
         	this.textboxStatus = 0;
		// remove event listeners
		document.documentElement.removeEventListener("keypress",this,false);
         	document.documentElement.removeEventListener("click",this,false);
         	// error correction
 	       	// set cursor invisible
         	cursor.setAttributeNS(null,"visibility","hidden");
          }

          if (evt.currentTarget.nodeName == "g")
          {
		// set textbox status
		this.textboxStatus = 1;
	 	// set event listeners
         	document.documentElement.addEventListener("keypress",this,false);
         	document.documentElement.addEventListener("click",this,false);
         	
         	// set cursor position, if zero
		if(textVal.charAt(0)=="0")
		{
			textVal = textVal.substring(1,textVal.length); // if click on textbox with content "0", delete the "0"
	 	}

	 	// set cursor visibility
         	cursor.setAttributeNS(null,"visibility","visible");
	  }
	  else
	  {
	  	this.textboxStatus = 2;
	  }
        }

	if (evt.type =="keypress")
	{
         var keynum = evt.charCode;
         var keychar = String.fromCharCode(keynum);
       	 if(keynum==8)
       	 {
       		textVal=textVal.substring(0,textVal.length-1); // delete text with backspace
       	 }
       	else if(keynum==13 || keynum==10) // press return (enter)
       	 {
         	// set cursor invisible
     		cursor.setAttributeNS(null,"visibility","hidden");
		// remove event listeners
     		document.documentElement.removeEventListener("keypress",this,false);
     		document.documentElement.removeEventListener("click",this,false);
       	 }
       	else
       	{       	textVal+=keychar;}     
        }

         textNode.firstChild.nodeValue=textVal;

	 // set cursor position, if visible
	 if(textVal.length>0)
	 {
		var cursorPos = this.x-this.boxWidth/2+textNode.getComputedTextLength()+3; // textlength in pixel
	 }
	 else
	 {
		var cursorPos = this.x-this.boxWidth/2+3;
	 }
         if (cursor.getAttributeNS(null,"visibility") == "visible")
         {
          cursor.setAttributeNS(null,"x1",cursorPos);
          cursor.setAttributeNS(null,"x2",cursorPos);
         }

	// set value of the control object
	this.getValue();		
         
}       

textbox.prototype.getValue = function() {
	
	// set value of the control object
	
	var textboxVal = document.getElementById(this.groupId+"Text").firstChild.nodeValue;
	
	if (typeof(this.controlObjectId) == "string") {
		this.controlObject = sliderArray[this.controlObjectId];//eval(this.controlObjectId);  
		this.controlObject.setValue(textboxVal);
	}
	return textboxVal;

}	

textbox.prototype.setValue = function(value) {
	document.getElementById(this.groupId+"Text").firstChild.nodeValue = value;
}
